@@ -1,6 +1,6 @@
 package Language::LispPerl::BuiltIns;
-$Language::LispPerl::BuiltIns::VERSION = '0.003';
-use Moo;
+$Language::LispPerl::BuiltIns::VERSION = '0.004';
+use Moose;
 
 use Carp;
 use Class::Load;
@@ -10,7 +10,9 @@ use Language::LispPerl::Printer;
 
 use Language::LispPerl::CoreBindings;
 
-use Role::Tiny qw//;
+
+use Moose::Util;
+#use Role::Tiny qw//;
 
 =head1 NAME
 
@@ -138,7 +140,7 @@ Usage (install Coroutine builtin functions):
 
 sub apply_role{
     my ($self, $role) = @_;
-    Role::Tiny->apply_roles_to_object( $self, $role );
+    Moose::Util::ensure_all_roles( $self, $role );
 }
 
 =head2 has_function
@@ -229,7 +231,7 @@ sub _impl_perlcall{
             and $ast->third()->type() eq "meta";
         $perl_func = \&{ $ns . "::" . $perl_func };
         my @rest = $ast->slice( ( defined $meta ? 3 : 2 ) .. $size - 1 );
-        unshift @rest, Language::LispPerl::Atom->new( "string", $ns )
+        unshift @rest, Language::LispPerl::Atom->new({ type =>  "string", value =>  $ns })
             if $blessed eq "->";
 
         return $self->evaler()->perlfunc_call( $perl_func, $meta, \@rest, $ast );
@@ -271,7 +273,7 @@ sub _impl_throw {
           . $msg->type() )
       if $msg->type() ne "string";
 
-    my $e = Language::LispPerl::Atom->new( "exception", $msg->value() );
+    my $e = Language::LispPerl::Atom->new({ type => "exception", value => $msg->value() });
     $e->{label}  = $label->value();
     $e->{caller} = $self->evaler()->copy_caller();
     $e->{pos} = $ast->{pos};
@@ -297,7 +299,7 @@ sub _impl_catch{
     if ($@) {
         my $e = $self->evaler()->exception();
         if ( !defined $e ) {
-            $e = Language::LispPerl::Atom->new( "exception", "unkown expection" );
+            $e = Language::LispPerl::Atom->new({ type => "exception", value => "unkown expection" } );
             $e->{label} = "undef";
             my @ec = ();
             $e->{caller} = \@ec;
@@ -309,7 +311,7 @@ sub _impl_catch{
         for ( ; $i > $saved_caller_depth ; $i-- ) {
             $self->evaler()->pop_caller();
         }
-        my $call_handler = Language::LispPerl::Seq->new("list");
+        my $call_handler = Language::LispPerl::Seq->new({ type => "list" });
         $call_handler->append($handler);
         $call_handler->append($e);
         $self->evaler()->clear_exception();
@@ -325,7 +327,7 @@ sub _impl_exception_label{
     $ast->error( "exception-label expects an exception as argument but got "
                      . $e->type() )
         if $e->type() ne "exception";
-    return Language::LispPerl::Atom->new( "string", $e->{label} );
+    return Language::LispPerl::Atom->new({ type => "string", value => $e->{label} });
 }
 
 sub _impl_exception_message{
@@ -336,7 +338,7 @@ sub _impl_exception_message{
         "exception-message expects an exception as argument but got "
             . $e->type() )
         if $e->type() ne "exception";
-    return Language::LispPerl::Atom->new( "string", $e->value() );
+    return Language::LispPerl::Atom->new({ type =>  "string", value => $e->value() } );
 }
 
 sub _impl_def{
@@ -381,7 +383,7 @@ sub _impl_def{
 
     $self->evaler()->new_var($name);
     my $value = $self->evaler()->_eval( $ast->fourth() );
-    $value->meta($meta);
+    $value->meta_data($meta);
     $self->evaler()->var($name)->value($value);
     return $value;
 }
@@ -476,7 +478,7 @@ sub _impl_fn{
         $i++;
     }
 
-    my $nast = Language::LispPerl::Atom->new( "function", $ast );
+    my $nast = Language::LispPerl::Atom->new({ type => "function", value => $ast });
 
     $nast->{context} = $self->evaler()->copy_current_scope();
 
@@ -503,7 +505,7 @@ sub _impl_apply{
         if $l->type() ne "list";
 
     # Build a ( <function> ( args list  ) ) list.
-    my $n = Language::LispPerl::Seq->new("list");
+    my $n = Language::LispPerl::Seq->new({ type => "list" });
     $n->append($f);
     foreach my $i ( @{ $l->value() } ) {
         $n->append($i);
@@ -534,7 +536,7 @@ sub _impl_defmacro{
             }
         $i++;
     }
-    my $nast = Language::LispPerl::Atom->new( "macro", $ast );
+    my $nast = Language::LispPerl::Atom->new({ type =>  "macro", value => $ast });
 
     $nast->{context} = $self->evaler()->copy_current_scope();
 
@@ -545,7 +547,7 @@ sub _impl_defmacro{
 sub _impl_gen_sym{
     my ($self, $ast, $symbol) = @_;
     $ast->error("gen-sym expects 0/1 argument") if $ast->size > 2;
-    my $s = Language::LispPerl::Atom->new("symbol");
+    my $s = Language::LispPerl::Atom->new({ type => "symbol" });
     if ( $ast->size() == 2 ) {
         my $pre = $self->evaler()->_eval( $ast->second() );
         $ast->("gen-sym expects string as argument")
@@ -586,7 +588,7 @@ sub _impl_list{
     my ($self, $ast) = @_;
     return $self->evaler()->empty_list() if $ast->size == 1;
     my @vs = $ast->slice( 1 .. $ast->size - 1 );
-    my $r  = Language::LispPerl::Seq->new("list");
+    my $r  = Language::LispPerl::Seq->new({ type => "list" });
     foreach my $i (@vs) {
         $r->append( $self->evaler()->_eval($i) );
     }
@@ -611,7 +613,7 @@ sub _impl_cdr{
         if $v->type() ne "list";
     return $self->evaler()->empty_list() if ( $v->size() == 0 );
     my @vs = $v->slice( 1 .. $v->size() - 1 );
-    my $r  = Language::LispPerl::Seq->new("list");
+    my $r  = Language::LispPerl::Seq->new({ type => "list"});
     $r->value( \@vs );
     return $r;
 }
@@ -627,7 +629,7 @@ sub _impl_cons{
     my @vs = ();
     @vs = $rvs->slice( 0 .. $rvs->size() - 1 ) if $rvs->size() > 0;
     unshift @vs, $fv;
-    my $r = Language::LispPerl::Seq->new("list");
+    my $r = Language::LispPerl::Seq->new({ type => "list" });
     $r->value( \@vs );
     return $r;
 }
@@ -738,7 +740,7 @@ sub _impl_num_bin{
 
     my $vv1 = $v1->value();
     my $vv2 = $v2->value();
-    return Language::LispPerl::Atom->new( "number", $num_func->( $vv1 , $vv2 ) * 1 );
+    return Language::LispPerl::Atom->new({ type =>  "number", value => $num_func->( $vv1 , $vv2 ) * 1 });
 }
 my $STRING_FUNCTIONS = {
     'eq' => sub{ shift() eq shift(); },
@@ -843,7 +845,7 @@ sub _impl_length{
     my ($self, $ast, $symbol) = @_;
     $ast->error("length expects 1 argument") if $ast->size() != 2;
     my $v = $self->evaler()->_eval( $ast->second() );
-    my $r = Language::LispPerl::Atom->new( "number", 0 );
+    my $r = Language::LispPerl::Atom->new({ type =>  "number", value => 0 });
     if ( $v->type() eq "string" ) {
         $r->value( length( $v->value() ) );
         return $r;
@@ -871,18 +873,18 @@ sub _impl_reverse{
     my $v = $self->evaler()->_eval( $ast->second() );
 
     if ( $v->type() eq "string" ) {
-        return Language::LispPerl::Atom->new( "string", scalar( reverse( $v->value() ) ) );
+        return Language::LispPerl::Atom->new({ type =>  "string", value => scalar( reverse( $v->value() ) ) });
     }
 
     if ( $v->type() eq "list" ) {
-        my $r = Language::LispPerl::Seq->new("list");
+        my $r = Language::LispPerl::Seq->new({ type => "list" });
         my @vv = reverse @{ $v->value() };
         $r->value( \@vv );
         return $r;
     }
 
     if ( $v->type() eq "vector" or $v->type() eq "xml" ) {
-        my $r = Language::LispPerl::Atom->new( $v->type() );
+        my $r = Language::LispPerl::Atom->new({ type =>  $v->type() });
         my @vv = reverse @{ $v->value() };
         $r->value( \@vv );
         return $r;
@@ -914,7 +916,7 @@ sub _impl_append{
 
     if ( $v1type eq "string" ) {
         # Concat strings.
-        return Language::LispPerl::Atom->new( "string", $v1->value() . $v2->value() );
+        return Language::LispPerl::Atom->new({ type =>  "string", value => $v1->value() . $v2->value() });
     }
 
     if ( $v1type eq "list" or $v1type eq "vector" ) {
@@ -922,16 +924,16 @@ sub _impl_append{
         push @r, @{ $v1->value() };
         push @r, @{ $v2->value() };
         if ( $v1type eq "list" ) {
-            return Language::LispPerl::Seq->new( "list", \@r );
+            return Language::LispPerl::Seq->new({ type => "list", value => \@r });
         }
         else {
-            return Language::LispPerl::Atom->new( "vector", \@r );
+            return Language::LispPerl::Atom->new({ type => "vector", value => \@r });
         }
     }
 
     # Not a string, a list or a vector. Must be a map (cause we checked typing earlier)
     my %r = ( %{ $v1->value() }, %{ $v2->value() } );
-    return Language::LispPerl::Atom->new( "map", \%r );
+    return Language::LispPerl::Atom->new({ type => "map", value => \%r });
 }
 
 sub _impl_xml_name{
@@ -941,7 +943,7 @@ sub _impl_xml_name{
     my $v = $self->evaler()->_eval( $ast->second() );
     $ast->error( "xml-name expects xml as argument but got " . $v->type() )
         if $v->type() ne "xml";
-    return Language::LispPerl::Atom->new( "string", $v->{name} );
+    return Language::LispPerl::Atom->new({ type => "string", value => $v->{name} });
 }
 
 sub _impl_keys{
@@ -953,9 +955,9 @@ sub _impl_keys{
         if $v->type() ne "map";
     my @r = ();
     foreach my $k ( keys %{ $v->value() } ) {
-        push @r, Language::LispPerl::Atom->new( "keyword", $k );
+        push @r, Language::LispPerl::Atom->new({ type => "keyword", value => $k });
     }
-    return Language::LispPerl::Seq->new( "list", \@r );
+    return Language::LispPerl::Seq->new({ type => "list", value => \@r });
 }
 
 sub _impl_namespace_begin{
@@ -984,14 +986,14 @@ sub _impl_object_id{
     my ($self, $ast) = @_;
     $ast->error("object-id expects 1 argument") if $ast->size != 2;
     my $v = $self->evaler()->_eval( $ast->second() );
-    return Language::LispPerl::Atom->new( "string", $v->object_id() );
+    return Language::LispPerl::Atom->new({ type =>  "string", value => $v->object_id() });
 }
 
 sub _impl_type{
     my ($self, $ast) = @_ ;
     $ast->error("type expects 1 argument") if $ast->size != 2;
     my $v = $self->evaler()->_eval( $ast->second() );
-    return Language::LispPerl::Atom->new( "string", $v->type() );
+    return Language::LispPerl::Atom->new({ type => "string", value => $v->type() } );
 }
 
 sub _impl_meta{
@@ -1006,9 +1008,9 @@ sub _impl_meta{
             "meta expects 1 meta data as the second arguments but got "
                 . $vm->type() )
             if $vm->type() ne "meta";
-        $v->meta($vm);
+        $v->meta_data($vm);
     }
-    my $m = $v->meta();
+    my $m = $v->meta_data();
     $ast->error( "no meta data in " . Language::LispPerl::Printer::to_string($v) )
         if !defined $m;
     return $m;
@@ -1027,7 +1029,7 @@ sub _impl_clj_string{
     my ($self, $ast) = @_;
     $ast->error("clj->string expects 1 argument") if $ast->size() != 2;
     my $v = $self->evaler()->_eval( $ast->second() );
-    return Language::LispPerl::Atom->new( "string", Language::LispPerl::Printer::to_string($v) );
+    return Language::LispPerl::Atom->new({ type => "string", value => Language::LispPerl::Printer::to_string($v) });
 }
 
 sub _impl_trace_vars{
@@ -1044,7 +1046,7 @@ sub _impl_perlobj_type{
     $ast->error( "perlobj-type expects perlobject as argument but got "
                      . $v->type() )
         if ( $v->type() ne "perlobject" );
-    return Language::LispPerl::Atom->new( "string", ref( $v->value() ) );
+    return Language::LispPerl::Atom->new({ type => "string", value => ref( $v->value() ) });
 }
 
 sub _impl_perl_clj{
@@ -1057,4 +1059,5 @@ sub _impl_perl_clj{
     return $self->evaler()->perl2clj( $o->value() );
 }
 
+__PACKAGE__->meta()->make_immutable();
 1;
