@@ -1,10 +1,6 @@
 package Language::LispPerl::Evaler;
-$Language::LispPerl::Evaler::VERSION = '0.003';
-use strict;
-use warnings;
-
-use Carp;
-use Moo;
+$Language::LispPerl::Evaler::VERSION = '0.004';
+use Moose;
 
 use File::ShareDir;
 use File::Spec;
@@ -38,16 +34,39 @@ has 'loaded_files' => ( is => 'ro', default => sub{ {}; } );
 has 'file_stack' => ( is => 'ro',  default => sub{ []; } );
 has 'caller' => ( is => 'ro' , default => sub{ []; } );
 
-has 'quotation_scope' => ( is => 'ro', default => sub{ 0; });
-has 'syntaxquotation_scope' => ( is => 'ro', default => sub{ 0; });
+has 'quotation_scope' => ( is => 'ro', default => 0 );
+has 'syntaxquotation_scope' => ( is => 'ro', default => 0 );
 
 has 'exception' => ( is => 'rw' );
 
 # The container for the builtin functions.
-has 'builtins' => ( is => 'ro', default => sub{
-                        my ($self) = @_;
-                        return Language::LispPerl::BuiltIns->new({ evaler => $self });
-                    });
+has 'builtins' => ( is => 'ro', lazy_build => 1 );
+
+sub _build_builtins{
+    my ($self) = @_;
+    return Language::LispPerl::BuiltIns->new({ evaler => $self });
+}
+
+sub to_hash{
+    my ($self) = @_;
+    return {
+        'scopes' => Language::LispPerl::Printer::to_perl( $self->scopes() ),
+        'loaded_files' => Language::LispPerl::Printer::to_perl( $self->loaded_files() ),
+        'file_stack' => Language::LispPerl::Printer::to_perl( $self->file_stack() ),
+        'caller' => Language::LispPerl::Printer::to_perl( $self->caller() ),
+        'quotation_scope' => Language::LispPerl::Printer::to_perl( $self->quotation_scope() ),
+        'syntaxquotation_scope' => Language::LispPerl::Printer::to_perl( $self->syntaxquotation_scope() ),
+        'exception' => Language::LispPerl::Printer::to_perl( $self->exception() ),
+        __class => $self->blessed(),
+    };
+}
+
+sub from_hash{
+    my ($class, $hash) = @_;
+    return $class->new({
+        map { $_ => Language::LispPerl::Reader::from_perl( $hash->{$_} ) } keys %$hash
+    });
+}
 
 =head2 new_instance
 
@@ -314,10 +333,10 @@ sub read {
     return $res;
 }
 
-our $empty_list = Language::LispPerl::Seq->new("list");
-our $true       = Language::LispPerl::Atom->new( "bool", "true" );
-our $false      = Language::LispPerl::Atom->new( "bool", "false" );
-our $nil        = Language::LispPerl::Atom->new( "nil", "nil" );
+our $empty_list = Language::LispPerl::Seq->new({ type => "list" });
+our $true       = Language::LispPerl::Atom->new({type =>  "bool", value => "true" });
+our $false      = Language::LispPerl::Atom->new({type =>  "bool", value => "false"});
+our $nil        = Language::LispPerl::Atom->new({type =>  "nil",  value => "nil"});
 
 sub true{ return $true; }
 sub false{ return $false; }
@@ -375,10 +394,10 @@ sub bind {
         return $nil;
     }
     elsif ( $type eq "accessor" ) {
-        return Language::LispPerl::Atom->new( "accessor", $self->bind($value) );
+        return Language::LispPerl::Atom->new({ type => "accessor", value => $self->bind($value) } );
     }
     elsif ( $type eq "sender" ) {
-        return Language::LispPerl::Atom->new( "sender", $self->bind($value) );
+        return Language::LispPerl::Atom->new({ type => "sender", value => $self->bind($value) });
     }
     elsif ( $type eq "syntaxquotation" or $type eq "quotation" ) {
         $self->{syntaxquotation_scope} += 1 if $type eq "syntaxquotation";
@@ -411,12 +430,12 @@ sub bind {
     elsif ( $type eq "symbol"
         and $self->{quotation_scope} > 0 )
     {
-        my $q = Language::LispPerl::Atom->new( "quotation", $value );
+        my $q = Language::LispPerl::Atom->new({ type => "quotation", value => $value });
         return $q;
     }
     elsif ( $class eq "Seq" ) {
         return $empty_list if $type eq "list" and $ast->size() == 0;
-        my $list = Language::LispPerl::Seq->new("list");
+        my $list = Language::LispPerl::Seq->new({ type => "list" });
         $list->type($type);
         foreach my $i ( @{$value} ) {
             if ( $i->type() eq "dequotation" and $i->value() =~ /^@/ ) {
@@ -529,7 +548,7 @@ sub _eval {
                 if ( $name eq "&" ) {
                     $i++;
                     $name      = $fargsvalue->[$i]->value();
-                    $rest_args = Language::LispPerl::Seq->new("list");
+                    $rest_args = Language::LispPerl::Seq->new({ type => "list" });
                     $self->new_var( $name, $rest_args );
                 }
                 else {
@@ -584,7 +603,7 @@ sub _eval {
                 if ( $name eq "&" ) {
                     $i++;
                     $name      = $fargsvalue->[$i]->value();
-                    $rest_args = Language::LispPerl::Seq->new("list");
+                    $rest_args = Language::LispPerl::Seq->new({ type => "list" });
                     $self->new_var( $name, $rest_args );
                 }
                 else {
@@ -618,7 +637,7 @@ sub _eval {
     }
     elsif ( $type eq "accessor" ) {
         my $av = $self->_eval($value);
-        my $a  = Language::LispPerl::Atom->new( "unknown", $av->value() );
+        my $a  = Language::LispPerl::Atom->new({ type => "unknown", value => $av->value() });
         my $at = $av->type();
         if ( $at eq "number" ) {
             $a->type("index accessor");
@@ -637,7 +656,7 @@ sub _eval {
         $ast->error( "sender expects a string or keyword but got " . $type )
           if $sn->type() ne "string"
           and $sn->type() ne "keyword";
-        my $s = Language::LispPerl::Atom->new( "symbol", $sn->value() );
+        my $s = Language::LispPerl::Atom->new({ type => "symbol", value => $sn->value() });
         return $self->bind($s);
     }
     elsif ( $type eq "symbol" ) {
@@ -650,7 +669,7 @@ sub _eval {
         return $self->bind($ast);
     }
     elsif ( $class eq "Seq" and $type eq "vector" ) {
-        my $v  = Language::LispPerl::Atom->new("vector");
+        my $v  = Language::LispPerl::Atom->new({ type => "vector" });
         my @vv = ();
         foreach my $i ( @{$value} ) {
             push @vv, $self->_eval($i);
@@ -659,7 +678,7 @@ sub _eval {
         return $v;
     }
     elsif ( $class eq "Seq" and ( $type eq "map" or $type eq "meta" ) ) {
-        my $m  = Language::LispPerl::Atom->new("map");
+        my $m  = Language::LispPerl::Atom->new({ type => "map" });
         my %mv = ();
         my $n  = scalar @{$value};
         $ast->error( $type . " should have even number of items" )
@@ -694,7 +713,7 @@ sub _eval {
           and $firsttype ne "string"
           and $firsttype ne "keyword";
         my @items = ();
-        my $xml = Language::LispPerl::Atom->new( "xml", \@items );
+        my $xml = Language::LispPerl::Atom->new({ type => "xml", value => \@items });
         $xml->{name} = $first->value();
         my @rest = $ast->slice( 1 .. $size - 1 );
         foreach my $i (@rest) {
@@ -708,7 +727,7 @@ sub _eval {
               and $it ne "meta"
               and $it ne "list";
             if ( $it eq "meta" ) {
-                $xml->meta($iv);
+                $xml->meta_data($iv);
             }
             elsif ( $it eq "list" ) {
                 foreach my $i ( @{ $iv->value() } ) {
@@ -902,7 +921,7 @@ sub clj2perl {
     elsif ( $type eq "function" ) {
         my $f = sub {
             my @args = @_;
-            my $cljf = Language::LispPerl::Seq->new("list");
+            my $cljf = Language::LispPerl::Seq->new({ type => "list" });
             $cljf->append($ast);
             foreach my $arg (@args) {
                 $cljf->append( $self->perl2clj($arg) );
@@ -922,7 +941,7 @@ sub wrap_perlobj {
     while ( ref($v) eq "REF" ) {
         $v = ${$v};
     }
-    return Language::LispPerl::Atom->new( "perlobject", $v );
+    return Language::LispPerl::Atom->new({ type => "perlobject", value => $v });
 }
 
 =head2 perl2clj
@@ -938,30 +957,30 @@ Usage:
 sub perl2clj {
     my ($self, $v) = @_;
     if ( !defined ref($v) or ref($v) eq "" ) {
-        return Language::LispPerl::Atom->new( "string", $v );
+        return Language::LispPerl::Atom->new({ type =>  "string", value => $v });
     }
     elsif ( ref($v) eq "SCALAR" ) {
-        return Language::LispPerl::Atom->new( "string", ${$v} );
+        return Language::LispPerl::Atom->new({ type =>  "string", value => ${$v} });
     }
     elsif ( ref($v) eq "HASH" ) {
         my %m = ();
         foreach my $k ( keys %{$v} ) {
             $m{$k} = $self->perl2clj( $v->{$k} );
         }
-        return Language::LispPerl::Atom->new( "map", \%m );
+        return Language::LispPerl::Atom->new({ type => "map", value => \%m });
     }
     elsif ( ref($v) eq "ARRAY" ) {
         my @a = ();
         foreach my $i ( @{$v} ) {
             push @a, $self->perl2clj($i);
         }
-        return Language::LispPerl::Atom->new( "vector", \@a );
+        return Language::LispPerl::Atom->new({ type =>  "vector", value => \@a });
     }
     elsif ( ref($v) eq "CODE" ) {
-        return Language::LispPerl::Atom->new( "perlfunction", $v );
+        return Language::LispPerl::Atom->new({ type => "perlfunction", value => $v });
     }
     else {
-        return Language::LispPerl::Atom->new( "perlobject", $v );
+        return Language::LispPerl::Atom->new({ type =>  "perlobject", value => $v });
 
         #$ast->error("expect a reference of scalar or hash or array");
     }
@@ -976,4 +995,5 @@ sub trace_vars {
     }
 }
 
+__PACKAGE__->meta->make_immutable();
 1;
