@@ -1,78 +1,45 @@
 package Language::LispPerl::Seq;
-$Language::LispPerl::Seq::VERSION = '0.003';
-use strict;
-use warnings;
+$Language::LispPerl::Seq::VERSION = '0.004';
+use Moose;
 
 use Language::LispPerl::Logger;
 use Language::LispPerl::Printer;
 
 our $id      = 0;
 
-sub new {
-    my $class = shift;
-    my $type  = shift;
-    $type = "list" if !defined $type;
-    my $value = shift;
-    my @seq   = ();
-    $value = \@seq if !defined $value;
-    my $self = {
-        class     => "Seq",
-        type      => $type,
-        value     => $value,
-        object_id => "seq" . ( $id++ ),
-        meta      => undef,
-        pos       => {
-            filename => "unknown",
-            line     => 0,
-            col      => 0
-        }
+has 'class' => ( is => 'ro', isa => 'Str', default => 'Seq' );
+has 'type' => ( is => 'rw', isa => 'Str', default => 'list' );
+has 'value' => ( is => 'rw', default => sub{ [] } );
+has 'object_id' => ( is => 'ro', isa => 'Str', default => sub{ 'seq'.( $id++ ); } );
+has 'meta_data' => ( is => 'rw' );
+has 'pos' => ( is => 'rw', isa => 'HashRef', default => sub{
+                   return {
+                       filename => "unknown",
+                       line     => 0,
+                       col      => 0
+                   }
+               });
+
+sub to_hash{
+    my ($self) = @_;
+    return {
+        class => $self->class(),
+        type => $self->type(),
+        value => Language::LispPerl::Printer::to_perl( $self->value() ),
+        object_id => $self->object_id(),
+        meta_data => Language::LispPerl::Printer::to_perl( $self->meta_data() ),
+        pos => Language::LispPerl::Printer::to_perl( $self->pos() ),
+        __class => $self->blessed(),
     };
-    bless $self;
-    return $self;
 }
 
-sub class {
-    my $self = shift;
-    return $self->{class};
+sub from_hash{
+    my ($class, $hash) = @_;
+    return $class->new({
+        map{ $_ => Language::LispPerl::Reader::from_perl( $hash->{$_} ) } keys %$hash
+    });
 }
 
-sub type {
-    my $self = shift;
-    my $type = shift;
-    if ( defined $type ) {
-        $self->{type} = $type;
-    }
-    else {
-        return $self->{type};
-    }
-}
-
-sub object_id {
-    my $self = shift;
-    return $self->{object_id};
-}
-
-sub meta {
-    my $self = shift;
-    my $meta = shift;
-    if ( defined $meta ) {
-        $self->{meta} = $meta;
-    }
-    else {
-        return $self->{meta};
-    }
-}
-
-sub value {
-    my $self  = shift;
-    my $value = shift;
-    if ( defined $value ) {
-        $self->{value} = $value;
-    }
-    else {
-        return $self->{value};
-    }
-}
 
 sub prepend {
     my $self = shift;
@@ -151,4 +118,6 @@ sub error {
     Language::LispPerl::Logger::error($msg);
 }
 
+
+__PACKAGE__->meta()->make_immutable();
 1;
